import * as v from 'valibot';
import {Resource} from '../common';

type IsCreated = boolean;

type TotalCount = number;

type DeleteResult = unknown;

export interface DataSource<
	ItemData extends object = object,
	Query extends object = object,
	ID extends string = string
> {
	initialize(): Promise<unknown>;
	getTotalCount?(query?: Query): Promise<TotalCount>;
	getMultiple(query?: Query): Promise<ItemData[]>;
	getById(id: ID): Promise<ItemData | null>;
	getSingle?(query?: Query): Promise<ItemData | null>;
	create(data: ItemData): Promise<ItemData>;
	delete(id: ID): Promise<DeleteResult>;
	emplace(id: ID, data: ItemData): Promise<[ItemData, IsCreated]>;
	patch(id: ID, data: Partial<ItemData>): Promise<ItemData | null>;
	prepareResource(resource: Resource): void;
	newId(): Promise<ID>;
}


export interface ResourceIdConfig<IdSchema extends v.BaseSchema> {
	generationStrategy: GenerationStrategy;
	serialize: (id: unknown) => string;
	deserialize: (id: string) => v.Output<IdSchema>;
	schema: IdSchema;
}

export interface GenerationStrategy {
	(dataSource: DataSource, ...args: unknown[]): Promise<string | number | unknown>;
}
