import {describe, afterAll, beforeAll, it} from 'vitest';
import {Application, application, resource, Resource, validation as v} from '../../src/common';
import {Backend, DataSource, RequestContext} from '../../src/backend';
import {createTestClient, DummyDataSource, dummyGenerationStrategy, TEST_LANGUAGE, TestClient} from '../utils';

const PORT = 3001;
const HOST = '127.0.0.1';
const BASE_PATH = '/api';
const ACCEPT = 'application/json';
const ACCEPT_LANGUAGE = 'en';
const ACCEPT_CHARSET = 'utf-8';
const CONTENT_TYPE_CHARSET = 'utf-8';
const CONTENT_TYPE = ACCEPT;

describe('decorators', () => {
	let Piano: Resource;
	let app: Application;
	let dataSource: DataSource;
	let backend: Backend;
	let server: ReturnType<Backend['createHttpServer']>;
	let client: TestClient;

	beforeAll(() => {
		Piano = resource(v.object(
			{
				brand: v.string()
			},
			v.never()
		))
			.name('Piano' as const)
			.route('pianos' as const)
			.id('id' as const, {
				generationStrategy: dummyGenerationStrategy,
				serialize: (id) => id?.toString() ?? '0',
				deserialize: (id) => Number.isFinite(Number(id)) ? Number(id) : 0,
				schema: v.number(),
			});

		app = application({
			name: 'piano-service',
		})
			.language(TEST_LANGUAGE)
			.resource(Piano);

		dataSource = new DummyDataSource();

		backend = app.createBackend({
			dataSource,
		});

		server = backend.createHttpServer({
			basePath: BASE_PATH
		});

		client = createTestClient({
			host: HOST,
			port: PORT,
		})
			.acceptMediaType(ACCEPT)
			.acceptLanguage(ACCEPT_LANGUAGE)
			.acceptCharset(ACCEPT_CHARSET)
			.contentType(CONTENT_TYPE)
			.contentCharset(CONTENT_TYPE_CHARSET);

		return new Promise((resolve, reject) => {
			server.on('error', (err) => {
				reject(err);
			});

			server.on('listening', () => {
				resolve();
			});

			server.listen({
				port: PORT
			});
		});
	});

	afterAll(() => new Promise<void>((resolve, reject) => {
		server.close((err) => {
			if (err) {
				reject(err);
			}

			resolve();
		});
	}));

	it('decorates requests', () => {
		server.requestDecorator((req) => {
			const reqMut = req as unknown as Record<string, unknown>;
			reqMut['foo'] = 'bar';
			return reqMut as unknown as RequestContext;
		});

		// TODO how to make assertions here
	});
});
