import { application, resource, validation as v } from '@modal-sh/yasumi';
import { http } from '@modal-sh/yasumi/backend';
import { randomUUID } from 'crypto';
import {JsonLinesDataSource} from '@modal-sh/yasumi-data-source-file-jsonl';
import { constants } from 'http2';

const UuidIdConfig = {
  generationStrategy: () => Promise.resolve(randomUUID()),
  schema: v.string(),
  serialize: (v: unknown) => v?.toString() ?? '',
  deserialize: (v: string) => v, // TODO resolve bytes/non-formatted UUIDs
};

const User = resource(
  v.object({
    email: v.string([v.email()]),
    password: v.string(),
    createdAt: v.datelike(),
    updatedAt: v.datelike(),
  })
)
  .name('User')
  .route('users')
  .id('id', UuidIdConfig)
  .canFetchItem()
  .canFetchCollection()
  .canCreate()
  .canEmplace()
  .canPatch()
  .canDelete();

const Post = resource(
  v.object({
    title: v.string(),
    content: v.string(),
  })
)
  .name('Post')
  .route('posts')
  .id('id', UuidIdConfig)
  .createdAt('createdAt')
  .updatedAt('updatedAt')
  .canFetchItem()
  .canFetchCollection()
  .canCreate()
  .canEmplace()
  .canPatch()
  .canDelete();

const app = application({
  name: 'cms'
})
  .resource(User)
  .resource(Post);

const backend = app.createBackend({
  dataSource: new JsonLinesDataSource(),
})
  .throwsErrorOnDeletingNotFound();

const server = backend.createHttpServer({
  basePath: '/api',
})
  .defaultErrorHandler((_req, res) => () => {
    throw new http.ErrorPlainResponse('urlNotFound', {
      statusCode: constants.HTTP_STATUS_NOT_FOUND,
      res,
    });
    // throw new http.ErrorPlainResponse('notImplemented', {
    //   statusCode: constants.HTTP_STATUS_NOT_IMPLEMENTED,
    //   res,
    // });
  });

server.listen(6969);
